'use strict';

const yeoman = require('yeoman-generator');
const path = require('path');
const fs = require('fs');
const chalk = require('chalk');
const yosay = require('yosay');
const _ = require('lodash');
const extend = require('deep-extend');
const selfupdate = require('selfupdate');
const packageJSON = require('../../package.json');
const notifier = require('node-notifier');
const camelCase = require('camelcase');
const Config = require('./config');

const GENERATOR_NAME = 'generator-gfe-lego';
const TEMPLATE_LEGO_WIDGET_REACT = 'lego-widget-react';
const TEMPLATE_LEGO_WIDGET_VUE = 'lego-widget-vue';
const TEMPLATE_LEGO_APPLICATION = 'lego-application';

/**
 * 读取git文件中的origin
 *
 * @return {String}
 */
function getGitOrigin() {
    let gitOrigin = '';
    try {
        let gitConfig = fs.readFileSync('./.git/config', 'utf-8');
        let m = gitConfig.match(/\[remote\s+"origin"]\s+url\s+=\s+(\S+)\s+/i);
        if (m) {
            gitOrigin = m[1];
        }
    } finally {
        return gitOrigin;
    }
}

/**
 * 提取repo中的homeUrl
 *
 * @param  {String} repo - repo地址
 * @return {String}
 */
function getHomeUrl(repo) {
    let url = '';
    try {
        let m = repo.match(/^git@(\S+)\.git$/i);
        if (m) {
            url = m[1].split(':').join('/');
        }
    } finally {
        return url;
    }
}

module.exports = yeoman.Base.extend({
    initializing() {
        this.props = {};
        // 自动更新
        selfupdate.isUpdated(packageJSON, (error, isUpdated) => {
            if (error) {
                return;
            }
            if (isUpdated) {
                return;
            }
            /**
             * 更新提示
             */
            notifier.notify({
                title: GENERATOR_NAME,
                subtitle: '已有新版本,正在更新...',
                message: `如果更新失败，请手动更新\n执行 npm i -g ${GENERATOR_NAME}`,
                contentImage: path.resolve(__dirname, 'peon.gif'),
                sound: true, // Only Notification Center or Windows Toasters
                wait: true // Wait with callback, until user action is taken against notification
            }, (err, response) => {
                // Response is response from notification
            });

            // 更新完毕
            selfupdate.update(packageJSON, (e, version) => {
                if (e) {
                    return;
                }
                notifier.notify({
                    title: GENERATOR_NAME,
                    subtitle: '更新完毕',
                    message: '请重新运行本应用\n执行 yo gfe-lego',
                    contentImage: path.resolve(__dirname, 'peon.gif'),
                    sound: true, // Only Notification Center or Windows Toasters
                    wait: true // Wait with callback, until user action is taken against notification
                }, (err, response) => {
                    // Response is response from notification
                });
            });
        });
    },
    /**
     * 与用户交互功能
     */
    prompting() {
        let done = this.async();

        // 用户提示
        this.log(yosay(
            'Welcome to the sweet ' + chalk.red(GENERATOR_NAME) + ' generator!'
        ));

        const prompts = [{
            name: 'boilerplate',
            type: 'list',
            choices: [TEMPLATE_LEGO_WIDGET_VUE, TEMPLATE_LEGO_WIDGET_REACT, TEMPLATE_LEGO_APPLICATION],
            default: TEMPLATE_LEGO_WIDGET_VUE,
            message: 'boilerplate'
        }, {
            name: 'name',
            message: 'Your project name',
            default: path.basename(process.cwd()) // Default to current folder name
        }, {
            name: 'version',
            default: '1.0.0',
            message: 'version'
        }, {
            name: 'description',
            default: 'f2e isomorphic general GB project',
            message: 'description'
        }, {
            name: 'repo',
            default: getGitOrigin(),
            message: 'git repository'
        }, {
            name: 'keywords',
            default: 'lego isomorphic',
            message: 'keywords',
            filter(words) {
                return words.split(/\s*,\s*/g);
            }
        }, {
            name: 'author',
            default: this.user.git.name(),
            message: 'author'
        }, {
            name: 'email',
            default: this.user.git.email(),
            message: 'E-Mail'
        }];

        this.prompt(prompts, function(props) {
            this.props = props;
            // To access props later use this.props.someAnswer;

            done();
        }.bind(this));
    },
    /**
     * 生成 LICENSE
     */
    default: function() {
        this.composeWith('license', {
            options: {
                name: this.props.author,
                email: this.props.email,
                website: ''
            }
        }, {
            local: require.resolve('generator-license/app')
        });
    },
    /**
     * 生成过程
     */
    writing: {
        init() {
            const map = {
                [TEMPLATE_LEGO_WIDGET_REACT]: TEMPLATE_LEGO_WIDGET_REACT,
                [TEMPLATE_LEGO_WIDGET_VUE]: TEMPLATE_LEGO_WIDGET_VUE,
                [TEMPLATE_LEGO_APPLICATION]: TEMPLATE_LEGO_APPLICATION
            };
            this.currentDir = map[this.props.boilerplate] || TEMPLATE_LEGO_WIDGET_REACT;
        },
        /**
         * 生成package.json
         */
        'package_json': function() {
            let currentPkg = this.fs.readJSON(this.destinationPath('package.json'), {});
            currentPkg.version = this.props.version;
            const allPkgJson = extend({}, {
                [TEMPLATE_LEGO_APPLICATION]: Config.APPLICATION_CONFIG,
                [TEMPLATE_LEGO_WIDGET_REACT]: Config.WIDGET_REACT_CONFIG,
                [TEMPLATE_LEGO_WIDGET_VUE]: Config.WIDGET_VUE_CONFIG
            });
            const selectedpkgJson = allPkgJson[this.props.boilerplate] || {};
            // 记录package
            this.pkg = extend({
                name: (this.props.boilerplate == TEMPLATE_LEGO_APPLICATION) ? _.kebabCase(this.props.name) : '@gfe/' + _.kebabCase(this.props.name),
                description: this.props.description,
                repository: {
                    type: 'git',
                    url: this.props.repo
                },
                author: {
                    name: this.props.author,
                    email: this.props.email
                },
                keywords: [],
                dependencies: selectedpkgJson.dependencies || {},
                devDependencies: selectedpkgJson.devDependencies || {},
                scripts: selectedpkgJson.scripts || {},
                legoconfig: selectedpkgJson.legoconfig || {},
                config: {
                    'unsafe-perm': true
                },
                bugs: {
                    url: 'http://' + getHomeUrl(this.props.repo) + '/issues'
                },
                homepage: 'http://' + getHomeUrl(this.props.repo)
            }, currentPkg);

            // Combine the keywords
            if (this.props.keywords) {
                this.pkg.keywords = _.uniq(this.props.keywords.concat(this.pkg.keywords));
            }

            if (selectedpkgJson.main) {
                this.pkg.main = selectedpkgJson.main;
            }

            if (selectedpkgJson.browser) {
                this.pkg.browser = selectedpkgJson.browser;
            }

            // Let's extend package.json so we're not overwriting user previous fields
            this.fs.writeJSON(this.destinationPath('package.json'), this.pkg);
        },
        /**
         * 生成cortex.json
         */
        'cortex_json': function() {
            let currentCortex = this.fs.readJSON(this.destinationPath('cortex.json'), {});

            let cortex = extend({
                name: this.pkg.name,
                version: this.pkg.version,
                description: this.pkg.description,
                main: 'index.js',
                directories: {
                    src: 'dist',
                    html: 'html'
                },
                repository: this.props.repo,
                author: this.pkg.author,
                license: this.pkg.license,
                keywords: this.pkg.keywords,
                bugs: {
                    url: 'http://' + getHomeUrl(this.props.repo) + '/issues'
                },
                homepage: 'http://' + getHomeUrl(this.props.repo)
            }, currentCortex);

            // Let's extend package.json so we're not overwriting user previous fields
            this.fs.writeJSON(this.destinationPath('cortex.json'), cortex);
        },
        /**
         * copy文件夹
         */
        'directories': function() {
            this.fs.copy(this.templatePath('./' + this.currentDir + '/static') + '/babelrc', this.destinationPath('./.babelrc'));
            this.fs.copy(this.templatePath('./' + this.currentDir + '/static') + '/eslintrc.json', this.destinationPath('./.eslintrc.json'));
            this.fs.copy(this.templatePath('./' + this.currentDir + '/static') + '/gitignore', this.destinationPath('./.gitignore'));
            this.fs.copy(this.templatePath('./' + this.currentDir + '/static') + '/npmignore', this.destinationPath('./.npmignore'));
            this.fs.copy(this.templatePath('./' + this.currentDir + '/static') + '/npmrc', this.destinationPath('./.npmrc'));
            this.fs.copy(this.templatePath('./' + this.currentDir + '/static') + '/**/*.*', this.destinationPath('./'));
            this.fs.copyTpl(this.templatePath('./' + this.currentDir + '/tpl') + '/**/*.*', this.destinationPath('./'), {
                PkgName: this.pkg.name,
                PkgDescription: this.pkg.description,
                AppName: this.pkg.name.replace('@gfe/', ''),
                AppNameCamel: camelCase(this.pkg.name.replace('@gfe/', ''))
            });
        }
    },
    /**
     * install任务
     */
    install() {
        let opt = {
            cwd: this.destinationPath('./')
        };
        this.spawnCommandSync('npm', ['install'], opt);
    }
});
