'use strict';
/*
    browser端的action打包配置,根据action里面引用的widget进行打包
 */
const fs = require('fs');
const _ = require('lodash');
const path = require('path');
const webpack = require('webpack');
const HappyPack = require('happypack');

const CortexRecombinerPlugin = require('cortex-recombiner-webpack-plugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin');

const actionParser = require('@gfe/lego-source-parser').action;
const configParser = require('@gfe/lego-source-parser').config;
const legoUtils = require('lego-utils');

const webpackUtil = require('../common/webpack.util.js');

const packageJSON = require('../../package.json');
const rootDir = path.dirname(path.resolve(__dirname, '../../package.json'));
const legoConfig = packageJSON.legoconfig;
const supportorPreProcessor = path.resolve(rootDir, legoConfig.supportorpreprocessor);
const resourceResolver = require('./resourceresolver.js');

// 非贪婪模式,将@xx/ 替换成xx-
const REGEX_NORMALIZA_PACKAGE_NAME = /@(.+?)\//g;

const f2eCiConfig = require('../../f2eci.json');
const DIST_PATH = f2eCiConfig.dist;

const env = f2eCiConfig.env;
const plugins = [
    new webpack.LoaderOptionsPlugin({
        options: {
            postcss() {
                // 处理css兼容性代码，无须再写-webkit之类的浏览器前缀
                return [
                    require('postcss-initial')({
                        reset: 'all' // reset only inherited rules
                    }),
                    require('autoprefixer')({
                        browsers: ['> 5%']
                    })
                ];
            }
        }
    }),
    new webpack.optimize.CommonsChunkPlugin({
        name: 'common',
        filename: 'common.js',
        minChunks: Infinity
    }),
    new ExtractTextPlugin({
        filename: '[name].css',
        disable: false,
        allChunks: true
    }),
    new CortexRecombinerPlugin({
        base: rootDir,
        noBeta: env === 'product'
    }),
    new HappyPack({
        cache: true,
        loaders: [{
            loader: 'babel-loader',
            options: {
                presets: ['es2015', 'react', 'stage-0'],
                cacheDirectory: true
            }
        }]
    })
];
if (env === 'product') {
    plugins.push(new webpack.DefinePlugin({
        'process.env': {
            NODE_ENV: JSON.stringify('production')
        }
    }));
    plugins.push(new webpack.optimize.UglifyJsPlugin({
        compress: {
            warnings: false
        }
    }));
}

/**
 * 生成action的入口
 *
 * @return {Object}
 */
function getActionEntries() {
    const actionDir = path.resolve(rootDir, legoConfig.actiondir);
    const actions = legoUtils.common.getFilteredFiles(actionDir, {
        nameRegex: /\.(action)\.(js)$/
    });

    const ret = {};
    actions.forEach((action) => {
        const fileContent = fs.readFileSync(action, 'UTF-8');
        const staticResources = actionParser.extractStaticResources(fileContent);
        if (!(staticResources && staticResources.length)) {
            console.warn(`${action} didn't has staticResources at all.`);
            return true;
        }
        staticResources.forEach((resource) => {
            resource = resourceResolver.transformStaticResourceToAbsolutePath(resource);
            if (!ret[resource]) {
                ret[resource] = [];
            }
            ret[resource].push(action);
        });
    });
    return ret;
}

/**
 * 生成静态的entry
 *
 * @return {Object}
 */
function getStaticEntries() {
    const entry = {};
    const staticDir = path.resolve(rootDir, legoConfig.staticjsdir);
    const entries = legoUtils.common.getFilteredFiles(staticDir, {
        nameRegex: /\.js$/
    });
    entries.forEach((file) => {
        const relativePath = path.relative(staticDir, file);
        const distAbsolutePath = resourceResolver.transfromSourceToAbsolutePath(relativePath);
        entry[distAbsolutePath] = [file];
    });
    return entry;
}

/**
 * 组合所有entry,主要是将action的内容合并到静态内容中
 *
 * @return {Object}
 */
function combineEntries() {
    const staticEntries = getStaticEntries();
    const actionEntries = getActionEntries();
    /*
        进行combo,每一个action的去寻找他们对应static的key,然后拼接
        洗key,然后通过处理成相对路径
        然后更新公用包
     */
    Object.keys(actionEntries).forEach((aboslutePath) => {
        if (!(staticEntries && staticEntries[aboslutePath])) return true;
        let rets = staticEntries[aboslutePath];
        if (_.isString(rets)) {
            rets = [rets];
        }
        rets = rets.concat(actionEntries[aboslutePath]);

        staticEntries[aboslutePath] = rets;
    });

    const entry = {};
    Object.keys(staticEntries).forEach((staticEntry) => {
        const key = resourceResolver.transformToRelativePath(staticEntry);
        entry[key] = staticEntries[staticEntry];
    });
    const commonChunkName = legoConfig.commonchunkname || 'common';
    entry[commonChunkName] = [supportorPreProcessor, '@gfe/lego'];
    return entry;
}

/**
 * 生成组件名称和路径的mapping关系
 *
 * @return {Object}
 */
function generateWidgetNameToPath() {
    // widgets声明的文件夹,f2eci.json这个文件内部配置的地址都是以它为相对路径的
    const widgetConfigPath = path.resolve(rootDir, legoConfig.widgetconfig);
    const widgets = require(widgetConfigPath).widgets || [];
    const entry = {};

    widgets.forEach((widget) => {
        // 有两种情况,一种是相对路径,一种是根据npm引用的路径
        const isRelativePath = widget.indexOf('.') === 0;
        let widgetPath;
        if (isRelativePath) {
            widgetPath = path.resolve(rootDir, widget);
        } else {
            widgetPath = require.resolve(widget);
        }

        const fileContent = fs.readFileSync(widgetPath, 'UTF-8');
        const widgetName = configParser.extractWidgetName(fileContent);
        if (!widgetName) return true;
        entry[widgetName] = widgetPath;
    });
    return entry;
}

module.exports = {
    entry: combineEntries(),
    output: {
        filename: '[name].js',
        path: resourceResolver.WEBPACK_DIST,
        publicPath: resourceResolver.PUBLIC_PATH,
        chunkFilename: '[name].[chunkhash].js'
    },
    cache: true,
    resolve: {
        // 添加后缀名顺序
        extensions: ['.js', '.jsx', '.vue', '.less']
    },
    externals: {
        react: 'this lego-react',
        redux: 'this lego-redux',
        'react-redux': 'this lego-react-redux',
        'react-dom': 'this lego-react-dom',
        'react-addons-css-transition-group': 'this lego-react-css-transition-group',
        vue: 'this lego-vue',
        vuex: 'this lego-vuex'
    },
    module: {
        rules: [{
            include: [supportorPreProcessor],
            loader: 'lego-supportor-preprocessor-loader'
        }, {
            test: /\.(action)\.(js)$/,
            exclude: /node_modules\/(?!@gfe)/,
            use: [{
                loader: 'lego-action-loader',
                options: {
                    widgetNameToPath: generateWidgetNameToPath()
                }
            }]
        }, {
            include: webpackUtil.covertToAllMatchedIncludes(webpackUtil.getWidgetPaths()),
            exclude: /node_modules\/(?!@gfe)/,
            use: [{
                loader: 'babel-loader'
            }, {
                loader: 'lego-config-loader',
                options: {
                    include: 'static'
                }
            }]
        }, {
            test: /\.(js)$/,
            exclude: /node_modules\/(?!(@gfe|lego))/,
            use: require.resolve('happypack/loader.js')

        }, {
            test: /\.(jsx)$/,
            exclude: /node_modules\/(?!@gfe)/,
            use: require.resolve('happypack/loader.js')
        }, {
            test: /\.vue$/,
            exclude: /node_modules\/(?!(@gfe))/,
            use: [{
                loader: 'vue-loader',
                options: {
                    loaders: {
                        less: ExtractTextPlugin.extract({
                            use: [{
                                loader: 'css-loader'
                            }, {
                                loader: 'postcss-loader'
                            }, {
                                loader: 'less-loader'
                            }],
                            fallback: 'vue-style-loader' // <- this is a dep of vue-loader, so no need to explicitly install if using npm3
                        }),
                        css: ExtractTextPlugin.extract({
                            use: 'css-loader',
                            fallback: 'vue-style-loader' // <- this is a dep of vue-loader, so no need to explicitly install if using npm3
                        })
                    }
                }
            }]
        }, {
            test: /\.css$/,
            use: ExtractTextPlugin.extract({
                use: [{
                    loader: 'css-loader'
                }],
                fallback: 'style-loader'
            })
        }, {
            test: /\.less$/,
            use: ExtractTextPlugin.extract({
                use: [{
                    loader: 'css-loader'
                }, {
                    loader: 'less-loader'
                }],
                fallback: 'style-loader'
            })
        }, {
            test: /\.(png|jpg|gif|svg|eot|ttf|woff|woff2)$/,
            loader: 'url-loader',
            options: {
                limit: 5000
            }
        }]
    },
    // 对于引用的包,是使用package里的browser信息还是main信息
    target: 'web',
    plugins,
    // devServer
    devServer: {
        contentBase: DIST_PATH,
        historyApiFallback: false,
        hot: true,
        port: 9074,
        publicPath: resourceResolver.PUBLIC_PATH,
        noInfo: false,
        proxy: {
            '/app/*': {
                target: 'http://localhost:9074',
                bypass(req) {
                    let pkgName = packageJSON.name;
                    pkgName = pkgName.replace(REGEX_NORMALIZA_PACKAGE_NAME, (word, p1) => {
                        return `${p1}-`;
                    });
                    return req.url.replace(`/app/${pkgName}`, '');
                }
            }
        }
    }
};