'use strict';
/*
    browser端的action打包配置,根据action里面引用的widget进行打包
 */
const fs = require('fs');
const path = require('path');
const HappyPack = require('happypack');

const CortexRecombinerPlugin = require('cortex-recombiner-webpack-plugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin');

const configParser = require('@gfe/lego-source-parser').config;

const packageJSON = require('../../package.json');
const rootDir = path.dirname(path.resolve(__dirname, '../../package.json'));
const supportorPreProcessor = path.resolve(rootDir, './supportorpreprocessor.js');

// 非贪婪模式,将@xx/ 替换成xx-
const REGEX_NORMALIZA_PACKAGE_NAME = /@(.+?)\//g;

const f2eCiConfig = require('../../f2eci.json');
const DIST_PATH = f2eCiConfig.dist;
const WEBPACK_DIST = path.join(rootDir, f2eCiConfig.dist, f2eCiConfig['static-src']);
const PUBLIC_PATH = f2eCiConfig.urlPrefix + f2eCiConfig['static-src'] + '/';

const env = f2eCiConfig.env;
const plugins = [
    new ExtractTextPlugin({
        filename: '[name].css',
        disable: false,
        allChunks: true
    }),
    new CortexRecombinerPlugin({
        base: rootDir,
        noBeta: env === 'product'
    }),
    new HappyPack({
        cache: true,
        loaders: [{
            loader: 'babel-loader',
            options: {
                presets: ['es2015', 'stage-0'],
                cacheDirectory: true
            }
        }]
    })
];

/**
 * 生成action的入口
 *
 * @return {Object}
 */
function getEntries() {
    return {
        index: [supportorPreProcessor, path.resolve(rootDir, './action/action.action')]
    };
}

/**
 * 生成组件名称和路径的mapping关系
 *
 * @return {Object}
 */
function generateWidgetNameToPath() {
    // widgets声明的文件夹,f2eci.json这个文件内部配置的地址都是以它为相对路径的
    const widgetConfigPath = path.resolve(rootDir, './config/widgets.js');
    const widgets = require(widgetConfigPath).widgets || [];
    const entry = {};

    widgets.forEach((widget) => {
        // 有两种情况,一种是相对路径,一种是根据npm引用的路径
        const isRelativePath = widget.indexOf('.') === 0;
        let widgetPath;
        if (isRelativePath) {
            widgetPath = path.resolve(rootDir, widget);
        } else {
            widgetPath = require.resolve(widget);
        }

        const fileContent = fs.readFileSync(widgetPath, 'UTF-8');
        const widgetName = configParser.extractWidgetName(fileContent);
        if (!widgetName) return true;
        entry[widgetName] = widgetPath;
    });
    return entry;
}

module.exports = {
    entry: getEntries(),
    output: {
        filename: '[name].js',
        path: WEBPACK_DIST,
        publicPath: PUBLIC_PATH,
        chunkFilename: '[name].[chunkhash].js'
    },
    cache: true,
    resolve: {
        // 添加后缀名顺序
        extensions: ['.js', '.jsx', '.vue', '.less']
    },
    externals: {
        react: 'this lego-react',
        redux: 'this lego-redux',
        'react-redux': 'this lego-react-redux',
        'react-dom': 'this lego-react-dom',
        'react-addons-css-transition-group': 'this lego-react-css-transition-group',
        vue: 'this lego-vue',
        vuex: 'this lego-vuex'
    },
    module: {
        rules: [{
            include: [supportorPreProcessor],
            loader: 'lego-supportor-preprocessor-loader'
        }, {
            test: /\.(action)$/,
            exclude: /node_modules\/(?!@gfe)/,
            use: [{
                loader: 'lego-action-loader',
                options: {
                    widgetNameToPath: generateWidgetNameToPath()
                }
            }]
        }, {
            include: [path.resolve(rootDir, './src/config.js')],
            exclude: /node_modules\/(?!@gfe)/,
            use: [{
                loader: 'babel-loader'
            }, {
                loader: 'lego-config-loader',
                options: {
                    include: 'static'
                }
            }]
        }, {
            test: /\.(js)$/,
            exclude: /node_modules\/(?!(@gfe|lego))/,
            use: require.resolve('happypack/loader.js')

        }, {
            test: /\.vue$/,
            exclude: /node_modules\/(?!(@gfe))/,
            use: [{
                loader: 'vue-loader',
                options: {
                    loaders: {
                        less: ExtractTextPlugin.extract({
                            use: [{
                                loader: 'css-loader'
                            }, {
                                loader: 'less-loader'
                            }],
                            fallback: 'vue-style-loader' // <- this is a dep of vue-loader, so no need to explicitly install if using npm3
                        }),
                        css: ExtractTextPlugin.extract({
                            use: 'css-loader',
                            fallback: 'vue-style-loader' // <- this is a dep of vue-loader, so no need to explicitly install if using npm3
                        })
                    }
                }
            }]
        }, {
            test: /\.css$/,
            use: ExtractTextPlugin.extract({
                use: [{
                    loader: 'css-loader'
                }],
                fallback: 'style-loader'
            })
        }, {
            test: /\.less$/,
            use: ExtractTextPlugin.extract({
                use: [{
                    loader: 'css-loader'
                }, {
                    loader: 'less-loader'
                }],
                fallback: 'style-loader'
            })
        }, {
            test: /\.(png|jpg|gif|svg|eot|ttf|woff|woff2)$/,
            loader: 'url-loader',
            options: {
                limit: 5000
            }
        }]
    },
    // 对于引用的包,是使用package里的browser信息还是main信息
    target: 'web',
    plugins,
    // devServer
    devServer: {
        contentBase: DIST_PATH,
        historyApiFallback: false,
        hot: true,
        port: 9074,
        publicPath: PUBLIC_PATH,
        noInfo: false,
        proxy: {
            '/app/*': {
                target: 'http://localhost:9074',
                bypass(req) {
                    let pkgName = packageJSON.name;
                    pkgName = pkgName.replace(REGEX_NORMALIZA_PACKAGE_NAME, (word, p1) => {
                        return `${p1}-`;
                    });
                    return req.url.replace(`/app/${pkgName}`, '');
                }
            }
        }
    }
};
